/***************************************************************/
/*                                                             */
/*                      SAMPLE data queue api program          */
/*                                                             */
/*    Dave Dilling                                             */
/***************************************************************/

#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <time.h>					//measure time to do loop

#include "cwbdq.h"
#include "cwbsv.h"				//serviceability

#define MAXDTALEN 100


VOID main(unsigned short argc, char * argv[], char * argp[])
{
	cwbDQ_Attr				queueAttrib;		
	cwbDQ_QueueHandle		queueHandle;
	cwbDQ_QueueHandle		queueHandle2;
	cwbDQ_Data				queueData;
	cwbSV_ErrHandle		hCwbErr;	
	char						szErrText[1024];
	long						lErrMsgLen;
	
	char			szSysName[]		= "";
	char			szDQName[]		= "TSTDQ";
	char			szLibName[]		= "QGPL";
	int			rc;	
	char			myKey[3];		//EBCDIC key
	char			myData[100];	//Data
	long			lWaitTime;		//wait time on read
	char			ch;				//dummy for delay

	if (argc == 2)
		lWaitTime = atol(argv[1]);
	else
		lWaitTime = 0;


	//create the error msg handle
	rc = cwbSV_CreateErrHandle(&hCwbErr);

	//********* set up all the attributes **************
	//create attribute object
	queueAttrib = cwbDQ_CreateAttr();
	if (queueAttrib == 0)
		goto dqerror;

	//set max record length of data
	rc = cwbDQ_SetMaxRecLen(queueAttrib, MAXDTALEN);
	if (rc != 0)
		goto dqerror;

	//set order to keyed
	rc = cwbDQ_SetOrder(queueAttrib, CWBDQ_SEQ_KEYED);
	if (rc != 0)
		goto dqerror;

	//set key size
	rc = cwbDQ_SetKeySize(queueAttrib, (unsigned short) 3);
	if (rc != 0)
		goto dqerror; 

	
	//******** create and open the queue  ****************
	//create the data queue - this starts and ends a job
	rc = cwbDQ_Create(szDQName, szLibName, 
							szSysName,
							queueAttrib,
							hCwbErr);
	if (rc != 0 && rc != CWBDQ_QUEUE_EXISTS)
		goto dqerror;
	
	//done with attributes (does NOT end the job)
	//rc = cwbDQ_DeleteAttr(queueAttrib);
	//if (rc != 0)
	//	goto dqerror;

	//open the queue
	rc = cwbDQ_Open(szDQName, szLibName, 
							szSysName,
							&queueHandle,
							hCwbErr);
	if (rc != 0)
		goto dqerror;
	printf("Created/Opened DQ %s/%s on system %s. hDQ %d\n",
			szLibName, szDQName, szSysName, queueHandle);
			

	//*************** write some data *******************
	//create a data object
	queueData = cwbDQ_CreateData();
	if (queueData == 0)
		goto dqerror;

	//set data
	rc = cwbDQ_SetData(queueData,
							(unsigned char *) "1234567890",
							10);
	if (rc != 0)
		goto dqerror;

	//set key
	memcpy(myKey, "\xF1\xF1\xF1", 3);
	rc = cwbDQ_SetKey(queueData,
							myKey,
							sizeof(myKey));
	if (rc != 0)
		goto dqerror; 
	
	//write an entry	
	rc = cwbDQ_Write(queueHandle,
							queueData,
							CWB_TRUE,
							hCwbErr);
	if (rc != 0)
		goto dqerror;
	printf("Wrote data to data queue\n");

	// PAUSE
	printf("Press any key to continue\n");
	ch = getchar();


	//*********** try to read it back  *************
	//rc = cwbDQ_DeleteData(queueData);
	//queueData = cwbDQ_CreateData();
	//if (queueData == 0)
	//	goto dqerror;

	rc = cwbDQ_Read(queueHandle,
						queueData,
						lWaitTime,
						hCwbErr);
	if (rc == CWBDQ_TIMED_OUT)
		printf("Time out waiting for data. Wait time %d\n", lWaitTime);
	else if (rc == 0)
		printf("successfully read data from queue. Wait time %d\n", lWaitTime);
	else
		goto dqerror;
	
	//Read the data
	rc = cwbDQ_GetData(queueData,
							 myData);							
	if (rc != 0)
		goto dqerror;

	//read key
	rc = cwbDQ_GetKey(queueData,
							myKey);
	if (rc != 0)
		goto dqerror;
	printf("\t      data %s\n", myData);
	
	//******** read with no data  ****************	
	memset(myData, '\x0', sizeof(myData));
	rc = cwbDQ_Read(queueHandle,
						queueData,
						lWaitTime,
						hCwbErr);
	if (rc == CWBDQ_TIMED_OUT)
		printf("Time out waiting for data. Wait time %d\n", lWaitTime);
	else if (rc == 0)
	{
		printf("successfully read data from queue. Wait time %d\n", lWaitTime);

		//Read the data
		rc = cwbDQ_GetData(queueData,
								 myData);							
		if (rc != 0)
			goto dqerror;

		//read key
		rc = cwbDQ_GetKey(queueData,
								myKey);
		if (rc != 0)
			goto dqerror;
		printf("\t      data %s, key %s\n",
				myData, myKey);
	}
	else
		goto dqerror;
		

//*************** write some more data *******************
	//set data
	rc = cwbDQ_SetData(queueData,
							(unsigned char *) "0987654321",
							10);
	if (rc != 0)
		goto dqerror;

	//set key
	memcpy(myKey, "\xF1\xF1\xF1", 3);
	rc = cwbDQ_SetKey(queueData,
							myKey,
							sizeof(myKey));
	if (rc != 0)
		goto dqerror;
	
	//write an entry	
	rc = cwbDQ_Write(queueHandle,
							queueData,
							CWB_TRUE,
							hCwbErr);
	if (rc != 0)
		goto dqerror;
	printf("Wrote data to data queue\n");

//*************** clear the data queue *******************
	//clear entry	
	rc = cwbDQ_Clear(queueHandle,
							NULL,				//*key
							0,				//
							hCwbErr);
	if (rc != 0)
		goto dqerror;
	printf("Cleared data queue\n");

//************************** Q2 *******************************
//******  open a second queue to see if it uses the same job

	//create the data queue
	rc = cwbDQ_Create("TSTDQ2", szLibName, 
							szSysName,
							queueAttrib,
							hCwbErr);
	if (rc != 0 && rc != CWBDQ_QUEUE_EXISTS)
		goto dqerror;

	rc = cwbDQ_Open("TSTDQ2", szLibName, 
							szSysName,
							&queueHandle2,
							hCwbErr);
	if (rc != 0)
		goto dqerror;

	//clear entry	
	rc = cwbDQ_Clear(queueHandle2,
							NULL,				//*key
							0,				//
							hCwbErr);
	if (rc != 0)
		goto dqerror;
	printf("Create/Open/Clear data queue2 %s/%s\n",
				"TSTDQ2", szLibName);

	rc = cwbDQ_Close(queueHandle);
//*********************** end Q2 ******************************

	//******   All done so delete and close *************
	rc = cwbDQ_DeleteData(queueData);
	//done with attributes
	rc = cwbDQ_DeleteAttr(queueAttrib);
	if (rc != 0)
		goto dqerror;

	//close the data queue handle
	rc = cwbDQ_Close(queueHandle2);
	
	// PAUSE
	printf("Press any key to end\n");
	ch = getchar();


return;
dqerror:
	//query error object here
	cwbSV_GetErrText(hCwbErr,
							szErrText,
							(long) sizeof(szErrText),
							&lErrMsgLen);
	if (strlen(szErrText) > 1)
		printf(szErrText);
	else
		printf("Error rc %d.  No associated error message", rc);

	rc = cwbDQ_Close(queueHandle);
	rc = cwbDQ_DeleteData(queueData);
	rc = cwbDQ_DeleteAttr(queueAttrib);
	return;
}